/*
* ============================================================================
*  Name        : maudiocontrol.h
*  Part of     : mccexample / mccexapp
*  Description : MAudioControl class declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef M_MAUDIOCONTROL_H
#define M_MAUDIOCONTROL_H

#include <e32std.h>

#include "tcontrolstate.h"

class MMccExEngineObserver;

/**
 * Declares an abstract audio control interface. Actual implementation(s) may
 * include multiple up- and/or downlink streams and their combinations.
 */
class MAudioControl
    {
public:
    
    /**
     * Prepares this audio control.
     * 
     * @pre EStateCreated == State()
     * @post EStatePrepared == State()
     * @leave KErrNotReady If the state change is not allowed. Standard
     *                     Symbian error codes are used in other cases.
     */
    virtual void PrepareAudioL() = 0;
    
    /**
     * Starts this audio control.
     * 
     * @pre EStatePrepared == State()
     * @post EStateStarted == State()
     * @leave KErrNotReady If the state change is not allowed. Standard
     *                     Symbian error codes are used in other cases.
     */
    virtual void StartAudioL() = 0;
    
    /**
     * Pauses this audio control.
     * 
     * @pre EStateStarted == State()
     * @post EStatePaused == State()
     * @leave KErrNotReady If the state change is not allowed. Standard
     *                     Symbian error codes are used in other cases.
     */
    virtual void PauseAudioL() = 0;
    
    /**
     * Resumes this audio control from paused state.
     * 
     * @pre EStatePaused == State()
     * @post EStateStarted == State()
     * @leave KErrNotReady If the state change is not allowed. Standard
     *                     Symbian error codes are used in other cases.
     */
    virtual void ResumeAudioL() = 0;
    
    /**
     * Stops/resets this audio control. If the current audio state is 
     * EStateStopped this call is a no-operation.
     * 
     * @post EStateStopped == State()
     * @leave KErrNotReady If the state change is not allowed. Standard
     *                     Symbian error codes are used in other cases.
     */
    virtual void StopAudioL() = 0;
    
    /**
     * Sets the current audio output volume.
     * 
     * @pre EStatePrepared == State()
     * @post VolumeL() == aVolume
     * @leave KErrArgument If the given argument is larger than returned by 
     *                     MaxVolumeL(). Otherwise standard Symbian error
     *                     codes are used.
     * 
     */
    virtual void SetVolumeL( TInt aVolume ) = 0;
    
    /**
     * Returns the current audio output volume.
     * 
     * @pre EStatePrepared == State()
     * @return TInt Current volume of audio output.
     */
    virtual TInt VolumeL() = 0;
    
    /**
     * Returns the maximum supported volume level of the audio output.
     * 
     * @pre EStatePrepared == State()
     * @return TInt Maximum supported volume level.
     */
    virtual TInt MaxVolumeL() = 0;
    
    /**
     * Enables or disables mic mute. This does not stop media packet flow
     * upstream but rather uses the underlying mic gain to control whether
     * silence or speech data is sent to upstream.
     * 
     * @param aMuteState ETrue if mic mute should be enabled, EFalse if it
     *                   should be disabled.
     */
    virtual void SetMuteStateL( TBool aMuteState ) = 0;
    
    /**
     * Sets the current audio routing (public/private). Argument must be
     * either KMccAudioToPrivate or KMccAudioToPublic.
     * Note that default audio routing is always KMccAudioToPublic.
     * 
     * @see mmccinterfacedef.h
     * @pre EStatePrepared == State()
     * @post AudioRoutingL() == aRouting
     * @leave KErrArgument If the argument is not none of the specified. 
     *                     In other cases standard Symbian error codes.
     * @param aRouting New audio routing to be applied for the local 
     *                 audio-output.
     */
    virtual void SetAudioRoutingL( TUint32 aRouting ) = 0;
    
    /**
     * Returns the current audio routing of local audio-output (public or 
     * private). Return value is either KMccAudioToPrivate or
     * KMccAudioToPublic depending on the current routing.
     * Note that default audio routing is always KMccAudioToPublic.
     * 
     * @pre EStatePrepared == State()
     * @leave Standard Symbian error codes.
     * @return TUint32 Current audio routing.
     */
    virtual TUint32 AudioRoutingL() = 0;
    
    /**
     * Returns the current state of this audio control.
     * 
     * @return TControlState Current audio control state.
     */
    virtual TControlState State() = 0;
    
    /**
     * Releases all the resources reserved by this audio control and deletes
     * this instance of audio control. After calling this function, the audio
     * control can go out of scope and MUST NOT be referenced anymore.
     */
    virtual void ReleaseAudioControl() = 0;
    
    };

#endif  // M_MAUDIOCONTROL_H
