/*
* ============================================================================
*  Name        : mccexengine.h
*  Part of     : mccexample / mccexengine
*  Description : CMccExEngine declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CMCCEXENGINE_H
#define C_CMCCEXENGINE_H

#include <e32base.h>
#include "tnetworksettings.h"
#include "tcodecsettings.h"

class CMccInterface;
class CMccCodecInformation;
class CMccEventHandler;
class MAudioControl;
class MVideoControl;
class TCameraInfo;
class MMccExEngineObserver;
class TVideoDeviceParameters;


/**
 * CMccExEngine implements an application engine for the Media Control API
 * example. This engine serves as the main API between the UI and Media
 * Control API. CMccExEngine hides the required setup & managemental functions
 * from the application under a more simple API.
 */
class CMccExEngine : public CBase
    {
public:
    
    /**
     * Destructor.
     */
    IMPORT_C virtual ~CMccExEngine();
    
    /**
     * Two-phased constructor.
     */
    IMPORT_C static CMccExEngine* NewL();
    
    /**
     * Two-phased constructor.
     */
    IMPORT_C static CMccExEngine* NewLC();
    
    /**
     * Creates loopback audio interface (including up- & downlink audio
     * streams) and returns a pointer to it.
     * 
     * Loopback audio control consists of up- and downlink audio streams which
     * use actual network stack (incl. RTP stack) for delivering audio from
     * handset mic to handset audio-output device (whether it is earpiece or
     * loudspeaker).
     * 
     * With this control user can easily test that basic audio streaming
     * using Multimedia Comms Controller is working properly.
     * 
     * @see MAudioControl
     * @param aObserver The observer for the control.
     * @return MAudioControl Loopback audiocontrol.
     */
    IMPORT_C MAudioControl* CreateLoopbackAudioControlL(
            MMccExEngineObserver& aObserver );
    
    /**
     * Creates loopback video interface (including up- & downlink video
     * streams) and returns a pointer to it.
     * 
     * Loopback video control consists of up- and downlink video streams which
     * use actual network stack (incl. RTP stack) for delivering video from
     * handset camera to handset screen.
     * 
     * With this control user can easily test that basic video streaming
     * using Multimedia Comms Controller is working properly.
     * 
     * @see MVideoControl
     * @see CCamera
     * @see TVideoDeviceParameters
     * @param aObserver The observer for the control.
     * @param aRect The screen rectangle.
     * @param aResolution Video resolution to use in video capture.
     * @param aDevParams Video device parameters.
     * @return MVideoControl Loopback videocontrol.
     */
    IMPORT_C MVideoControl* CreateLoopbackVideoControlL(
            MMccExEngineObserver& aObserver,
            const TRect& aRect,
            const TSize& aResolution,
            const TVideoDeviceParameters& aDevParams );
    
    /**
     * Resets the engine. This function deletes and re-creates MCC and the
     * related resources.
     * 
     * @leave Standard Symbian OS error code.
     */
    IMPORT_C void ResetEngineL();
    
    /**
     * Returns the number of cameras supported by the handset. Function
     * queries this information from MCC and not from CCamera.
     * 
     * @leave Standard Symbian OS error code.
     * @return TInt The number of cameras supported.
     */
    IMPORT_C TInt SupportedCamerasL();
    
    /**
     * Returns the camera information associated for a specific camera.
     * Function queries this information from MCC and not from CCamera.
     * 
     * @leave Standard Symbian OS error code.
     * @param aCamera Index of the camera whose information to return. Note
     *                that this argument must be in the range of
     *                0 - (SupportedCamerasL() - 1).
     * @param aInfo Camera information class which will contain the related
     *              information upon return.
     */
    IMPORT_C void CameraInformationL( TInt aCamera, TCameraInfo& aInfo );
    
    /**
     * Returns the current network settings of the engine. These settings
     * are used for all sessions.
     * 
     * @param aSettings Upon return aSettings will contain the current 
     *                  network settings.
     */
    IMPORT_C void NetworkSettings( TNetworkSettings& aSettings ) const;
    
    /**
     * Sets the current network settings to the user specified ones. Note that
     * the caller is fully responsible for these settings being correct.
     * Note also that the settings are NOT dynamically applied to ongoing
     * sessions/streams and so forth.
     * 
     * @param aSettings New network settings to be used.
     */
    IMPORT_C void SetNetworkSettings( const TNetworkSettings& aSettings );
    
    /**
     * Returns the audio codec settings. After calling this function user has
     * a copy of the audio codec settings in preference order. If user
     * updates these values, no checks are made to ensure that the updated
     * values are correct.
     * 
     * @param aCodecs Array containing audio codec information classes.
     */
    IMPORT_C void AudioCodecSettingsL(
        RArray<TCodecSettings>& aCodecs ) const;
    
    /**
     * Set the audio codec settings. After calling this the first audio codec
     * in the given array is used as the codec for a loopback audio control.
     * 
     * @param aCodecs Array containing audio codec information.
     */
    IMPORT_C void SetAudioCodecSettingsL(
        const RArray<TCodecSettings>& aCodecs );
    
private:
    
    /**
     * Constructor for performing 1st stage construction.
     */
    CMccExEngine();

    /**
     * Symbian default constructor for performing 2nd stage construction.
     */
    void ConstructL();
    
    /**
     * Filters capability array based on the aMediaType. E.g. filter array so
     * it contains only audio capabilities by giving KUidMediaTypeAudio as an
     * argument to aMediaType.
     * 
     * @param aCapabilities Array of capabilities to filter.
     * @param aMediaType Media type which against to filter.
     */
    void FilterCapabilities(
            RPointerArray<CMccCodecInformation>& aCapabilities,
            const TUid& aMediaType ) const;
    
private:    // Data
    
    /**
     * MCC interface.
     * Own
     */
    CMccInterface* iInterface;
    
    /**
     * Event handler for MCC events.
     * Own
     */
    CMccEventHandler* iEventHandler;
    
    /**
     * Network settings.
     */
    TNetworkSettings iNwSettings;
    
    /**
     * Array containing the codec settings in preference order.
     */
    RArray<TCodecSettings> iCodecSettings;
    
    };

#endif // C_CMCCEXAMPLEAPPENGINE_H
