/*
* ============================================================================
*  Name        : mccexappaudiocontainer.h
*  Part of     : mccexample / mccexapp
*  Description : CMccExAppAudioContainer declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CMCCEXAPPAUDIOCONTAINER_H
#define C_CMCCEXAPPAUDIOCONTAINER_H

#include "mccexappbasecontrol.h"

class CAknNavigationControlContainer;
class CAknNavigationDecorator;
class CEikGlobalTextEditor;
class MAudioControl;

/**
 * CMccExAppAudioContainer provides a CMccExAppBaseControl/CCoeControl based
 * container control for MCC example application audio view.
 */
class CMccExAppAudioContainer : public CMccExAppBaseControl
    {
public:

    /**
     * NewL. Two-phased constructor.
     * Create a CMccExAppAudioContainer object, which will draw itself to
     * aRect.
     * 
     * @param aRect The rectangle this view will be drawn to.
     * @param aNaviPane Navigation pane
     * @return A pointer to the created instance of CMccExAppAudioContainer.
     */
    static CMccExAppAudioContainer* NewL( const TRect& aRect,
            CAknNavigationControlContainer& aNaviPane );

    /**
     * NewLC. Two-phased constructor. Leaves created object in cleanup stack.
     * Create a CMccExAppAudioContainer object, which will draw itself to
     * aRect.
     * 
     * @param aRect Rectangle this view will be drawn to.
     * @param aNaviPane Navigation pane
     * @return A pointer to the created instance of CMccExAppAudioContainer.
     */
    static CMccExAppAudioContainer* NewLC( const TRect& aRect,
            CAknNavigationControlContainer& aNaviPane );

    /**
     * ~CMccExAppAudioContainer Destructor.
     */
    virtual ~CMccExAppAudioContainer();
    
// New functions
    
    /**
     * Sets the audio control and activates the volume indicator.
     * 
     * @leave Standard Symbian OS error code.
     * @param aControl Audio control.
     */
    void SetAudioControlAndActivateL( MAudioControl* aControl );
    
    /**
     * Removes the audio control and de-activates the volume indicator.
     * 
     * @leave Standard Symbian OS error code.
     */
    void DeactivateAndRemoveAudioControlL();
    
    /**
     * Updates the information shown to user. Function constructs a text line
     * from given resource.
     * 
     * @param aResource Resource ID of the string.
     */
    void UpdateInformationL( TInt aResource );
    
    /**
     * Updates the information shown to user. Function constructs a text line
     * from given resource and error code.
     * 
     * @param aResource Resource ID of the string.
     * @param aError Error code to append to the string.
     */
    void UpdateInformationL( TInt aResource, TInt aError );
    
    /**
     * Updates the information shown to user. This function does two things:
     * 1) Firstly it associates the first resource with the error code, e.g
     * "Error -33 detected -"
     * 2) Then it appends the secondary resource to the text resulting in
     * e.g. "Error -33 detected - state is prepared".
     * 
     * Note that it is the callers responsibility to be sure that the 
     * resource ID's are correct.
     * 
     * @param aResource Resource ID of the primary string.
     * @param aSecResource Resource ID of the secondary string.
     * @param aError Error code to append to the first string.
     */
    void UpdateInformationL( TInt aResource, TInt aSecResource,
            TInt aError );

// from CCoeControl
    
    /**
     * From CCoeControl.
     * Key event handling.
     * 
     * @see CCoeControl
     */
    TKeyResponse OfferKeyEventL( const TKeyEvent& aKeyEvent,
            TEventCode aType );
    
    /**
     * From CCoeControl.
     * Draw this CMccExAppAudioContainer to the screen.
     * 
     * @see CCoeControl
     * @param aRect the rectangle of this view that needs updating.
     */
    void Draw( const TRect& aRect ) const;
    
    /**
     * From CCoeControl.
     * Called by framework when the view size is changed.
     * 
     * @see CCoeControl
     */
    void SizeChanged();
    
    /**
     * From CCoeControl.
     * Returns count of components in this container.
     * 
     * @see CCoeControl
     * @return TInt Count of components.
     */
    TInt CountComponentControls() const;
    
    /**
     * From CCoeControl.
     * Returns specific component by index.
     * 
     * @see CCoeControl
     * @param aIndex Index of component to return.
     * @return CCoeControl* Component specified by index.
     */
    CCoeControl* ComponentControl( TInt aIndex ) const;
    
private:

    /**
     * ConstructL 2nd phase constructor.
     * Performs the second phase construction of a CMccExAppAudioContainer
     * object.
     * 
     * @param aRect The rectangle this view will be drawn to.
     */
    void ConstructL( const TRect& aRect );

    /**
     * CMccExAppAudioContainer. C++ default constructor.
     */
    CMccExAppAudioContainer( CAknNavigationControlContainer& aNaviPane );
    
// New functions
    
    /**
     * Handles the volume update related actions.
     */
    void HandleVolumeUpdateL();
    
    /**
     * Scales the UI volume levels (0-10) to HW volume levels (0-x).
     * 
     * @return TInt HW scaled volume level.
     */
    TInt ScaleVolumeToHwLimits( TInt aVolume ) const;
    
    /**
     * Scales the HW volume levels (0-x) to UI volume levels (0-10).
     * 
     * @return TInt UI scaled volume level.
     */
    TInt ScaleVolumeToUiLimits( TInt aVolume ) const;
    
    /**
     * Updates the editor text contents. Line end character is also supplied
     * by the caller and can be e.g. CEditableText::ELineBreak or 
     * CEditableText::ESpace
     * 
     * @see txtetext.h For line-end characters (in class CEditableText).
     * @param aText Text to be added to the editor.
     * @param aEndChar End of line character.
     */
    void UpdateEditorL( const TDesC& aText, TInt aEndChar  );
    
private:    // Data
    
    /**
     * Current navipane.
     */
    CAknNavigationControlContainer& iNaviPane;
    
    /**
     * Reference to audio control object.
     * Not own
     */
    MAudioControl* iAudioControl;
    
    /**
     * Decorator owning the actual volume control.
     * Own
     */
    CAknNavigationDecorator* iNaviDecorator;
    
    /**
     * Text editor for logging.
     * Own
     */
    CEikGlobalTextEditor* iEditor;
    
    /**
     * Maximum volume supported by the HW audio control.
     */
    TInt iMaxHWVolume;
    
    /**
     * Current HW scaled volume.
     */
    TInt iCurrentHwVolume;
    
    };

#endif // C_CMCCEXAPPAUDIOCONTAINER_H
