/*
* ============================================================================
*  Name        : mccexappaudiocontainer.cpp
*  Part of     : mccexample / mccexapp
*  Description : MCC example application CMccExAppAudioContainer
*                implementation.
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.1.1
*/

#include <eikspane.h>
#include <eikgted.h>
#include <txtglobl.h>
#include <aknappui.h>
#include <aknnavi.h>
#include <aknnavide.h>
#include <aknutils.h>
#include <aknsdrawutils.h>
#include <avkon.rsg>
#include <stringloader.h>

#include "mccexappaudiocontainer.h"
#include "maudiocontrol.h"
#include "mccexpanic.pan"

// Maximum volume in the CAknVolumeControl used, 0 - 10 is the allowed range
// for the volume control steps.
const TInt KMaxVolume( 10 );

const TInt KEditorDistanceFromScreenEdge( 8 );
const TInt KEditorMaxLength( 512 );
const TInt KEditorFlags( EEikEdwinNoAutoSelection |
                         EEikEdwinJustAutoCurEnd |
                         EEikEdwinLineCursor |
                         EEikEdwinNoHorizScrolling );
const TInt KFontColorIndexBlue( 200 );
const TInt KEditorBorderGrow( 2 );


// ======== LOCAL FUNCTIONS ========


// ---------------------------------------------------------------------------
// Returns height of font in pixels.
// ---------------------------------------------------------------------------
//
static TInt FontHeightInPixels()
    {
    const CFont* font = 
        AknLayoutUtils::FontFromId( EAknLogicalFontSecondaryFont );
    
    return font->HeightInPixels();
    }


// ---------------------------------------------------------------------------
// Returns total height of font in pixels (including line spacing).
// ---------------------------------------------------------------------------
//
static TInt FontTotalHeightInPixels()
    {
    const CFont* font =
        AknLayoutUtils::FontFromId( EAknLogicalFontSecondaryFont );
    
    return ( font->HeightInPixels() +
             font->AscentInPixels() +
             font->DescentInPixels() );
    }


// ---------------------------------------------------------------------------
// Returns total number lines of text fitting into specified rectangle.
// ---------------------------------------------------------------------------
//
static TInt NumOfFittingEditorLines( const TRect& aRect )
    {
    return aRect.Height() / FontTotalHeightInPixels();
    }


// ---------------------------------------------------------------------------
// Get appropriate layout for editor.
// ---------------------------------------------------------------------------
//
static AknLayoutUtils::SAknLayoutTextMultiline GetEditorLayout(
    const TRect& aRect )
    {
    /*
    SAknLayoutTextMultiline struct:
    
    @param iFont      font id, ELatinPlain12, ELatinBold12, ELatinBold13 etc..
    @param iC         colour index, 0..255
    @param iL         left margin
    @param iR         right margin
    @param iB         Baseline from top of the parent rectangle
    @param iW         text width in pixels
    @param iJ         justification. ELayoutAlignNone; ELayoutAlignCenter;
                      ELayoutAlignLeft; ELayoutAlignRight; ELayoutAlignBidi
                      
    @param iNumberOfLinesShown
    @param iNextLineB baseline of 2nd line for multi-line labels/editors
    */
    
    const TInt fontHeight( FontHeightInPixels() );
    AknLayoutUtils::SAknLayoutTextMultiline editorLayout;
    editorLayout.iFont = EAknLogicalFontSecondaryFont;
    editorLayout.iC = KFontColorIndexBlue;
    editorLayout.iL = KEditorDistanceFromScreenEdge;
    editorLayout.iR = KEditorDistanceFromScreenEdge;
    editorLayout.iB = KEditorDistanceFromScreenEdge + fontHeight;
    editorLayout.iW = aRect.Width() - editorLayout.iL - editorLayout.iR;
    editorLayout.iJ = ELayoutAlignLeft;
    editorLayout.iNumberOfLinesShown = NumOfFittingEditorLines( aRect );
    editorLayout.iNextLineB = editorLayout.iB + FontTotalHeightInPixels();
    
    return editorLayout;
    }


// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::NewL
// ---------------------------------------------------------------------------
//
CMccExAppAudioContainer* CMccExAppAudioContainer::NewL( const TRect& aRect,
    CAknNavigationControlContainer& aNaviPane )
    {
    CMccExAppAudioContainer* self =
        CMccExAppAudioContainer::NewLC( aRect, aNaviPane );
    CleanupStack::Pop( self );
    return self;
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::NewLC
// ---------------------------------------------------------------------------
//
CMccExAppAudioContainer* CMccExAppAudioContainer::NewLC( const TRect& aRect,
    CAknNavigationControlContainer& aNaviPane )
    {
    CMccExAppAudioContainer* self =
        new ( ELeave ) CMccExAppAudioContainer( aNaviPane );
    CleanupStack::PushL( self );
    self->ConstructL( aRect );
    return self;
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::ConstructL
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::ConstructL( const TRect& aRect )
    {
    CMccExAppBaseControl::CreateWindowAndBackgroundL( aRect );
    
    iNaviDecorator = iNaviPane.CreateVolumeIndicatorL(
                R_AVKON_NAVI_PANE_VOLUME_INDICATOR );
    
    TRect editorRect( aRect );
    editorRect.Shrink( KEditorDistanceFromScreenEdge,
                       KEditorDistanceFromScreenEdge );
    
    iEditor = new ( ELeave ) CEikGlobalTextEditor();
    iEditor->ConstructL( this, NumOfFittingEditorLines( editorRect ),
                         KEditorMaxLength, KEditorFlags,
                         EGulFontControlAll, EGulAllFonts );
    
    iEditor->SetContainerWindowL( *this );
    iEditor->CreateScrollBarFrameL();
    iEditor->ScrollBarFrame()->SetScrollBarVisibilityL(
        CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto );
    
    CMccExAppBaseControl::SetRectAndActivateL( aRect );
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::CMccExAppAudioContainer
// ---------------------------------------------------------------------------
//
CMccExAppAudioContainer::CMccExAppAudioContainer(
    CAknNavigationControlContainer& aNaviPane ) : iNaviPane( aNaviPane ),
    iMaxHWVolume( KMaxVolume )
    {
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::~CMccExAppAudioContainer
// ---------------------------------------------------------------------------
//
CMccExAppAudioContainer::~CMccExAppAudioContainer()
    {
    delete iNaviDecorator;
    delete iEditor;
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::SetAudioControlAndActivateL
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::SetAudioControlAndActivateL(
    MAudioControl* aControl )
    {
    // Assert always the basic preconditions that this object is in correct
    // state.
    __ASSERT_ALWAYS( aControl, User::Leave( KErrArgument ) );
    __ASSERT_ALWAYS( !iAudioControl, User::Leave( KErrAlreadyExists ) );
    __ASSERT_ALWAYS( iNaviDecorator, User::Leave( KErrNotReady ) );
    
    iAudioControl = aControl;
    
    // Adjust the minimum and maximum values before setting the actual value.
    // This because the audio min/max and current values can be anything and
    // by default volume control has 10 steps.
    CAknVolumeControl* volCtrl( static_cast<CAknVolumeControl*>( 
            iNaviDecorator->DecoratedControl() ) );
    
    iMaxHWVolume = iAudioControl->MaxVolumeL();
    volCtrl->SetRange( 0, KMaxVolume );
    iCurrentHwVolume = ScaleVolumeToUiLimits( iAudioControl->VolumeL() );
    volCtrl->SetValue( iCurrentHwVolume );
    iNaviPane.PushL( *iNaviDecorator );
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::DeactivateAndRemoveAudioControlL
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::DeactivateAndRemoveAudioControlL()
    {
    // Assert always the basic preconditions that this object is in correct
    // state.
    __ASSERT_ALWAYS( iAudioControl && iNaviDecorator,
            User::Leave( KErrNotReady ) );
    
    iAudioControl = NULL;
    iNaviPane.Pop( iNaviDecorator );
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::UpdateInformationL
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::UpdateInformationL( TInt aResource )
    {
    HBufC* msg = StringLoader::LoadLC( aResource );
    UpdateEditorL( *msg, CEditableText::ELineBreak );
    CleanupStack::PopAndDestroy( msg );
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::UpdateInformationL
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::UpdateInformationL( TInt aResource,
    TInt aError )
    {
    HBufC* msg = StringLoader::LoadLC( aResource, aError );
    UpdateEditorL( *msg, CEditableText::ELineBreak );
    CleanupStack::PopAndDestroy( msg );
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::UpdateInformationL
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::UpdateInformationL( TInt aResource,
    TInt aSecResource, TInt aError )
    {
    // Do not do any resource ID checks, caller is responsible.
    HBufC* msg = StringLoader::LoadLC( aResource, aError );
    UpdateEditorL( *msg, CEditableText::ESpace );
    CleanupStack::PopAndDestroy( msg );
    
    HBufC* secText = StringLoader::LoadLC( aSecResource );
    UpdateEditorL( *secText, CEditableText::ELineBreak );
    CleanupStack::PopAndDestroy( secText );
    }

// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::OfferKeyEventL
// ---------------------------------------------------------------------------
//
TKeyResponse CMccExAppAudioContainer::OfferKeyEventL(
    const TKeyEvent& aKeyEvent, TEventCode aType )
    {
    TKeyResponse resp( EKeyWasNotConsumed );
    if ( iAudioControl )
        {
        resp = iNaviDecorator->DecoratedControl()->OfferKeyEventL(
                aKeyEvent, aType );
        
        if ( EKeyWasConsumed == resp )
            {
            HandleVolumeUpdateL();
            }
        }
    
    if( EEventKeyUp == aType && EKeyWasNotConsumed == resp )
        {
        switch( aKeyEvent.iScanCode )
            {
            case EStdKeyUpArrow:
                iEditor->MoveCursorL( TCursorPosition::EFPageUp, EFalse );
                resp = EKeyWasConsumed;
                break;
            case EStdKeyDownArrow:
                iEditor->MoveCursorL( TCursorPosition::EFPageDown, EFalse );
                resp = EKeyWasConsumed;
                break;
            default:
                resp = EKeyWasNotConsumed;
                break;
            }
        }
    
    return resp;
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::Draw
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::Draw( const TRect& aRect ) const
    {
    CMccExAppBaseControl::Draw( aRect );
    
    CWindowGc& gc = SystemGc();
    
    gc.SetBrushStyle( CGraphicsContext::ENullBrush );
    gc.SetPenStyle( CGraphicsContext::ESolidPen );
    gc.SetPenColor( KRgbBlack );
    
    TRect editorRect( iEditor->Rect() );
    editorRect.Grow( KEditorBorderGrow, KEditorBorderGrow );
    gc.DrawRect( editorRect );
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::SizeChanged
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::SizeChanged()
    {
    TRect editorRect( Rect() );
    editorRect.Shrink( KEditorDistanceFromScreenEdge,
                       KEditorDistanceFromScreenEdge );
    
    AknLayoutUtils::LayoutEdwin( iEditor, editorRect, 
            GetEditorLayout( editorRect ) );
    
    CMccExAppBaseControl::SizeChanged();
    }


// ---------------------------------------------------------------------------
// CMccExAppMainContainer::CountComponentControls
// ---------------------------------------------------------------------------
//
TInt CMccExAppAudioContainer::CountComponentControls() const
    {
    return 1;
    }


// ---------------------------------------------------------------------------
// CMccExAppMainContainer::ComponentControl
// ---------------------------------------------------------------------------
//
CCoeControl* CMccExAppAudioContainer::ComponentControl( TInt aIndex ) const
    {
    switch ( aIndex )
        {
        case 0:
            return iEditor;
            break;
        default:
            MccExPanic( EMccExOutOfBounds );
            return NULL;
            break;
        }
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::HandleVolumeUpdateL
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::HandleVolumeUpdateL()
    {
    // Assert always the basic preconditions that this object is in correct
    // state.
    __ASSERT_ALWAYS( iNaviDecorator && iAudioControl,
            MccExPanic( EMccExNullPtr ) );
    
    TInt volume = static_cast<CAknVolumeControl*>(
            iNaviDecorator->DecoratedControl() )->Value();
    
    volume = ScaleVolumeToHwLimits( volume );
    if ( iCurrentHwVolume != volume )
        {
        iAudioControl->SetVolumeL( volume );
        iCurrentHwVolume = volume;
        }
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::ScaleVolumeToHwLimits
// ---------------------------------------------------------------------------
//
TInt CMccExAppAudioContainer::ScaleVolumeToHwLimits( TInt aVolume ) const
    {
    TInt ret = aVolume * ( iMaxHWVolume / KMaxVolume );
    return ret;
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::ScaleVolumeToUiLimits
// ---------------------------------------------------------------------------
//
TInt CMccExAppAudioContainer::ScaleVolumeToUiLimits( TInt aVolume ) const
    {
    // "Fast path", HW and UI max volume levels match
    TInt ret = KMaxVolume;
    if ( KMaxVolume == iMaxHWVolume )
        {
        ret = aVolume;
        }
    else
        {
        // "Slow path"
        if ( aVolume )
            {
            const TInt scaleFactor( iMaxHWVolume / KMaxVolume );
            ret = aVolume / scaleFactor;
            }
        else
            {
            ret = 0;
            }
        }
    
    return ret;
    }


// ---------------------------------------------------------------------------
// CMccExAppAudioContainer::UpdateEditor
// ---------------------------------------------------------------------------
//
void CMccExAppAudioContainer::UpdateEditorL( const TDesC& aText,
    TInt aEndChar )
    {
    const TChar lineBreak( aEndChar );
    CGlobalText* globalText = iEditor->GlobalText();
    
    globalText->InsertL( globalText->DocumentLength(), aText );
    globalText->InsertL( globalText->DocumentLength(), lineBreak );
    
    TCharFormat charFormat;
    TCharFormatMask charMask;
    
    iEditor->ApplyCharFormatL( charFormat, charMask );
    iEditor->MoveCursorL( TCursorPosition::EFPageDown, EFalse );
    DrawDeferred();
    }

