/*
* ============================================================================
*  Name        : loopbackvideocontrol.h
*  Part of     : mccexample / mccexengine
*  Description : CLoopbackVideoControl declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CLOOPBACKVIDEOCONTROL_H
#define C_CLOOPBACKVIDEOCONTROL_H

#include <e32base.h>

#include "mvideocontrol.h"
#include "tnetworksettings.h"

class CMccInterface;
class CMccEventHandler;
class CMccNetworkSession;
class CVideoStreamBase;
class CStreamStateMachine;
class TVideoDeviceParameters;

/**
 * CLoopbackVideoControl provides MVideoControl interface implementation.
 * 
 * It contains up- and downlink streams which transfer video from video input
 * to video output (from camera to screen) using the whole multimedia comms
 * controller subsystem.
 */
NONSHARABLE_CLASS( CLoopbackVideoControl ) : public CBase,
    public MVideoControl
    {
public:
    
    /**
     * Destructor.
     */
    virtual ~CLoopbackVideoControl();
    
    /**
     * Two-phased constructor.
     * 
     * @param aMcc Reference to MCC.
     * @param aEventHandler Reference to event handler.
     * @param aObserver Reference to the engine observer.
     * @param aRect Initial video screen rectangle.
     * @param aResolution Video resolution to use.
     * @param aParams Video device parameters.
     * @param aSettings Network settings.
     */
    static CLoopbackVideoControl* NewL( CMccInterface& aMcc,
            CMccEventHandler& aEventHandler,
            MMccExEngineObserver& aObserver,
            const TRect& aRect,
            const TSize& aResolution,
            const TVideoDeviceParameters& aParams,
            const TNetworkSettings& aSettings );
    
    /**
     * Two-phased constructor. Leaves the created instance into the cleanup
     * stack.
     * 
     * @param aMcc Reference to MCC.
     * @param aEventHandler Reference to event handler.
     * @param aObserver Reference to the engine observer.
     * @param aRect Initial video screen rectangle.
     * @param aResolution Video resolution to use.
     * @param aParams Video device parameters.
     * @param aSettings Network settings.
     */
    static CLoopbackVideoControl* NewLC( CMccInterface& aMcc,
            CMccEventHandler& aEventHandler,
            MMccExEngineObserver& aObserver,
            const TRect& aRect,
            const TSize& aResolution,
            const TVideoDeviceParameters& aParams,
            const TNetworkSettings& aSettings );
    
// from MVideoControl
    
    /**
     * From MVideoControl.
     * Prepares this video control.
     * 
     * @see mvideocontrol.h
     */
    void PrepareVideoL();
    
    /**
     * From MVideoControl.
     * Starts this video control.
     * 
     * @see mvideocontrol.h
     */
    void StartVideoL();
    
    /**
     * From MVideoControl.
     * Pauses this video control.
     * 
     * @see mvideocontrol.h
     */
    void PauseVideoL();
    
    /**
     * From MVideoControl.
     * Resumes this video control.
     * 
     * @see mvideocontrol.h
     */
    void ResumeVideoL();
    
    /**
     * From MVideoControl.
     * Stops this video control.
     * 
     * @see mvideocontrol.h
     */
    void StopVideoL();
    
    /**
     * From MVideoControl.
     * Deletes this video control.
     * 
     * @see mvideocontrol.h
     */
    void ReleaseVideoControl();
    
    /**
     * From MVideoControl.
     * Returns the state of this video control.
     * 
     * @see mvideocontrol.h
     */
    TControlState State();
    
    /**
     * From MVideoControl.
     * Updates the video screen rectangle.
     * 
     * @see mvideocontrol.h
     */
    void UpdateScreenRectL( const TRect& aRect,
            TInt aWindowOrdinalPosition,
            TInt aWindowOrdinalPriority,
            TInt aScreenRotation );
    
private:
    
    /**
     * Constructor for performing 1st stage construction.
     * 
     * @param aMcc Reference to MCC.
     * @param aEventHandler Reference to event handler.
     * @param aSettings Network settings.
     */
    CLoopbackVideoControl( CMccInterface& aMcc,
            CMccEventHandler& aEventHandler,
            const TNetworkSettings& aSettings );
    
    /**
     * Symbian default constructor for performing 2nd stage construction.
     * 
     * @param aObserver Reference to the engine observer.
     * @param aRect Initial video screen rectangle.
     * @param aResolution Video resolution to use.
     * @param aParams Video device parameters.
     */
    void ConstructL( MMccExEngineObserver& aObserver,
            const TRect& aRect,
            const TSize& aResolution,
            const TVideoDeviceParameters& aParams );
    
    /**
     * Helper function for finding a specific type codec. Function returns the
     * first matching codec or leaves with KErrNotFound.
     * 
     * @leave KErrNotFound If the codec is not found from aCodecs.
     * @param aType Type of the codec to search.
     * @param aCodecs Array of codecs where to search.
     * @return CMccCodecInformation Pointer to the found codec.
     */
    CMccCodecInformation* FindCodecL( const TUid& aType,
            const RPointerArray<CMccCodecInformation>& aCodecs );
    
private:    // Data
    
    /**
     * Reference to MCC.
     */
    CMccInterface& iMcc;
    
    /**
     * Reference to event handler.
     */
    CMccEventHandler& iEventHandler;
    
    /**
     * Network session.
     * Own
     */
    CMccNetworkSession* iSession;
    
    /**
     * State machine for streams.
     * Own
     */
    CStreamStateMachine* iStateMachine;
    
    /**
     * Array of video streams.
     * Own
     */
    RPointerArray<CVideoStreamBase> iVideoStreams;
    
    /**
     * Network settings.
     */
    TNetworkSettings iNwSettings;
    
    };

#endif  // C_CLOOPBACKVIDEOCONTROL_H
