/*
* ============================================================================
*  Name        : mcceventhandler.h
*  Part of     : mccexample / mccexample
*  Description : CMccEventHandler declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CMCCEVENTHANDLER_H
#define C_CMCCEVENTHANDLER_H

#include <e32base.h>
#include <mmccctrlobserver.h>

class CMccNetworkSession;
typedef RPointerArray<CMccNetworkSession> RSessionArray;

/**
 * CMccEventHandler implements the MMccCtrlObserver interface for observing
 * different MCC events.
 */
NONSHARABLE_CLASS( CMccEventHandler ) : public CBase, public MMccCtrlObserver
    {
public:
    
    /**
     * Destructor.
     */
    virtual ~CMccEventHandler();
    
    /**
     * Two-phased constructor.
     */
    static CMccEventHandler* NewL();
    
    /**
     * Two-phased constructor. Leaves the created instance into the cleanup
     * stack.
     */
    static CMccEventHandler* NewLC();
    
    /**
     * Adds an session for which the event handler can send events once they
     * occur. Once the session has been successfully added, it can be removed
     * by calling RemoveSession().
     * 
     * @see RemoveSession.
     * @leave KErrArgument If aSession == NULL.
     * @leave KErrAlreadyExists If the given session is already in array.
     * @leave Standard Symbian OS error code in other cases.
     * @param aSession Session to add as an event target.
     */
    void AddSessionL( CMccNetworkSession* aSession );
    
    /**
     * Removes a session which was added as an event target with AddSessionL.
     * If the given session is not found, then this function does nothing.
     * Note that this function does not delete the given network session.
     * 
     * @see AddSessionL.
     * @param aSession Session to remove from being an event target.
     */
    void RemoveSession( CMccNetworkSession* aSession );

// From MMccCtrlObserver
        
    /**
     * From MMccCtrlObserver.
     * Callback function to receive media events.
     * 
     * @see mmccctrlobserver.h
     */
    void MccEventReceived( const TMccEvent& aEvent );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to receive media started event.
     * 
     * @see mmccctrlobserver.h
     */
    void MccMediaStarted( TUint32 aSessionId,
                          TUint32 aLinkId,     
                          TUint32 aStreamId,
                          TUint32 aSinkSourceId );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to receive media stopped event.
     * 
     * @see mmccctrlobserver.h
     */
    void MccMediaStopped( TUint32 aSessionId,
                          TUint32 aLinkId,
                          TUint32 aStreamId,
                          TUint32 aSinkSourceId );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to receive media paused event.
     * 
     * @see mmccctrlobserver.h
     */
    void MccMediaPaused( TUint32 aSessionId,
                         TUint32 aLinkId,
                         TUint32 aStreamId,
                         TUint32 aSinkSourceId );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to receive media resumed event.
     * 
     * @see mmccctrlobserver.h
     */
    void MccMediaResumed( TUint32 aSessionId,
                          TUint32 aLinkId,
                          TUint32 aStreamId,
                          TUint32 aSinkSourceId );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to receive media prepared event.
     * 
     * @see mmccctrlobserver.h
     */
    void MccMediaPrepared( TUint32 aSessionId,
                           TUint32 aLinkId,
                           TUint32 aStreamId,
                           TUint32 aSinkSourceId );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to receive media inactivity event.
     * 
     * @see mmccctrlobserver.h
     */
    void MccMediaInactive( TUint32 aSessionId,
                           TUint32 aLinkId,
                           TUint32 aStreamId,
                           TUint32 aSinkSourceId );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to receive media activity event. The function is 
     * called when the first RTP packet is received by a downlink stream.
     * 
     * @see mmccctrlobserver.h
     */
    void MccMediaActive( TUint32 aSessionId,
                         TUint32 aLinkId,
                         TUint32 aStreamId,
                         TUint32 aSinkSourceId );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to receive error event.
     * 
     * @see mmccctrlobserver.h
     */
    void MccCtrlError( TInt aError,
                       TUint32 aSessionId,
                       TUint32 aLinkId,
                       TUint32 aStreamId,
                       TUint32 aSinkSourceId );
    
    /**
     * From MMccCtrlObserver.
     * Callback function to notify client of unknown media packet. Called when
     * packet with unknown payload type is received.
     * 
     * @see mmccctrlobserver.h
     */
    void UnknownMediaReceived( TUint32 aSessionId,
                               TUint32 aLinkId,
                               TUint32 aStreamId,
                               TUint32 aSinkSourceId,
                               TUint8 aPayloadType );
    
private:
    
    /**
     * Constructor for performing 1st stage construction.
     */
    CMccEventHandler();

    /**
     * Symbian default constructor for performing 2nd stage construction.
     */
    void ConstructL();
    
    /**
     * Finds a network session from internal array based on the given session
     * and link ID's. Function returns NULL if it does not find a matching
     * session.
     * 
     * @param aSessionId Session ID.
     * @param aLinkId Link ID.
     * @return CMccNetworkSession Pointer to found network session or NULL.
     */
    CMccNetworkSession* FindMatchingSession( TUint32 aSessionId,
            TUint32 aLinkId ) const;

private:    // Data
    
    /**
     * Array of sessions. Instances are not owned so they are not destroyed by
     * this class.
     */
    RSessionArray iSessionArray;
    
    };

#endif // C_CMCCEVENTHANDLER_H
