/*
* ============================================================================
*  Name        : mccnetworksession.h
*  Part of     : mccexample / mccexengine
*  Description : CMccNetworkSession declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CMCCNETWORKSESSION_H
#define C_CMCCNETWORKSESSION_H

#include <e32base.h>

class CMccInterface;
class TMccEvent;
class CStreamBase;
class TInetAddr;
class MMccExEngineObserver;


/**
 * CMccNetworkSession encapsulates a MCC session and a link. Once an instance
 * of CMccNetworkSession is constructed, media streams can be created into
 * it.
 */
NONSHARABLE_CLASS( CMccNetworkSession ) : public CBase
    {
public:

    /**
     * Destructor.
     */
    virtual ~CMccNetworkSession();

    /**
     * Two-phased constructor.
     * 
     * @param aMcc Reference to the MCC.
     * @param aObserver Reference to the engine observer.
     * @param aMediaQos Media QoS value.
     * @param aPort Local RTP port to use.
     */
    static CMccNetworkSession* NewL( CMccInterface& aMcc,
            MMccExEngineObserver& aObserver, TInt aMediaQos, TInt aPort );

    /**
     * Two-phased constructor. Leaves the created instance into the cleanup
     * stack.
     * 
     * @param aMcc Reference to the MCC.
     * @param aObserver Reference to the engine observer.
     * @param aMediaQos Media QoS value.
     * @param aPort Local RTP port to use.
     */
    static CMccNetworkSession* NewLC( CMccInterface& aMcc,
            MMccExEngineObserver& aObserver, TInt aMediaQos, TInt aPort );
    
    /**
     * Tests whether this session has the given session and link identifiers.
     * Returns ETrue if the given session and link identifiers match, EFalse
     * in other cases. Used by the event handler to identify network sessions
     * when distributing MCC events to different sessions.
     * 
     * @param aSessionId Session ID.
     * @param aLinkId Link ID.
     * @return TBool ETrue if session and link IDs match, EFalse otherwise.
     */
    TBool Matches( TUint32 aSessionId, TUint32 aLinkId ) const;
    
    /**
     * MCC event handler function. Caller should check first that the event
     * is really targeted to this session by using Matches() function.
     * 
     * @see Matches()
     * @param aEvent Event targeted to this session.
     */
    void HandleEvent( const TMccEvent& aEvent );
    
    /**
     * Returns the MCC session identifier of this network session.
     * 
     * @return TUint32 MCC session identifier.
     */
    TUint32 SessionId() const;
    
    /**
     * Returns the MCC link identifier of this network session.
     * 
     * @return TUint32 MCC link identifier.
     */
    TUint32 LinkId() const;
    
    /**
     * Adds a stream into this session. Ownership is NOT transferred.
     * 
     * @leave Standard Symbian OS error code if the addition fails.
     * @param aStream Stream to be added to this session.
     */
    void AddStreamL( CStreamBase& aStream );
    
    /**
     * Sets the remote address of this session.
     * 
     * @leave KErrNotReady If session or link has not been created and if the
     *                     link is not up yet.
     * @leave Standard Symbian OS error code.
     * @param aAddress Remote address to be set.
     */
    void SetRemoteAddressL( const TInetAddr& aAddress );
    
private:
    
    /**
     * Constructor for performing 1st stage construction.
     * 
     * @param aMcc Reference to the MCC.
     * @param aObserver Reference to the engine observer.
     * @param aMediaQos Media QoS value.
     * @param aPort Local RTP port to use.
     */
    CMccNetworkSession( CMccInterface& aMcc,
            MMccExEngineObserver& aObserver, TInt aMediaQos, TInt aPort );

    /**
     * Symbian default constructor for performing 2nd stage construction.
     */
    void ConstructL();
    
    /**
     * Finds a stream with the given stream ID from the internal array.
     * Returns NULL if such a stream is not found.
     * 
     * @param aStreamId Stream ID which against to search.
     * @return CStreamBase* Pointer to the found stream or NULL.
     */
    CStreamBase* FindStream( TUint32 aStreamId );

private:    // Data
    
    /**
     * Reference to MCC interface.
     */
    CMccInterface& iMcc;
    
    /**
     * Reference to the engine observer.
     */
    MMccExEngineObserver& iObserver;
    
    /**
     * MCC session identifier.
     */
    TUint32 iSessionId;
    
    /**
     * MCC link identifier.
     */
    TUint32 iLinkId;
    
    /**
     * Flag to indicate link creation success.
     */
    TBool iLinkUp;
    
    /**
     * Error code which is associated to link creation event. If link has not
     * yet been created, iErrorCode == KErrNone && iLinkUp == EFalse. In error
     * cases the combination might be iErrorCode == KErrNotReady && 
     * iLinkUp == EFalse. Successfull link creation has KErrNone && ETrue as
     * the combination.
     */
    TInt iErrorCode;
    
    /**
     * Array for streams associated to this network session. Note that the
     * streams are NOT owned by this session and it does not delete them.
     */
    RPointerArray<CStreamBase> iStreams;
    
    /**
     * Media QoS value.
     */
    TInt iMediaQos;
    
    /**
     * Local port.
     */
    TInt iLocalPort;
    
    };

#endif  // C_CMCCNETWORKSESSION_H
