/*
* ============================================================================
*  Name        : streambase.h
*  Part of     : mccexample / mccexengine
*  Description : CStreamBase declaration
*  Version     : %version: %
*
*  Copyright  2008-2009 Nokia.  All rights reserved.
*  This material, including documentation and any related computer
*  programs, is protected by copyright controlled by Nokia.  All
*  rights are reserved.  Copying, including reproducing, storing,
*  adapting or translating, any or all of this material requires the
*  prior written consent of Nokia.  This material also contains
*  confidential information which may not be disclosed to others
*  without the prior written consent of Nokia.
* ============================================================================
* Template version: 4.2
*/

#ifndef C_CSTREAMBASE_H
#define C_CSTREAMBASE_H

#include <e32base.h>
#include "tcontrolstate.h"

class CStreamStateMachine;
class CMccInterface;
class CMccCodecInformation;
class CStreamStateBase;

/**
 * CStreamBase provides a base class for all actual stream classes
 */
NONSHARABLE_CLASS( CStreamBase ) : public CBase
    {
public:

    /**
     * Destructor.
     */
    virtual ~CStreamBase();
    
    /**
     * Sets the RTCP usage status.
     * 
     * @param aRtcpUsage ETrue or EFalse whether RTCP is used.
     */
    void SetRtcpUsage( TBool aRtcpUsage );
    
    /**
     * Returns the current RCTP usage status.
     * 
     * @return TBool ETrue or EFalse whether RCTP is used. 
     */
    TBool RtcpUsage() const;
    
    /**
     * Returns the type of this stream.
     * 
     * @see mmccinterfacedef.h For possible stream types.
     * @return TInt Type of this stream.
     */
    TInt StreamType() const;
    
    /**
     * Link up event notification. This is called when the link is ready and
     * the stream can start making state changes. This function finalizes the
     * stream creation.
     * 
     * return TInt Standard Symbian OS error code if the internal stream
     *             creation fails.
     */
    TInt LinkReady();
    
    /**
     * Returns the session ID associated for this stream.
     * 
     * @return TUint32 Session identifier.
     */
    TUint32 SessionId() const;
    
    /**
     * Returns the link ID associated for this stream.
     * 
     * @return TUint32 Link identifier.
     */
    TUint32 LinkId() const;
    
    /**
     * Returns the stream ID associated for this stream. Returned stream ID is
     * KMaxTUint32 if the stream is not yet created.
     * 
     * @return TUint32 Stream identifier.
     */
    TUint32 StreamId() const;
    
    /**
     * Returns the sink endpoint ID associated for this stream. 
     * Returned endpoint ID is KMaxTUint32 if the sink is not yet added.
     * 
     * @return TUint32 Sink endpoint identifier.
     */
    TUint32 SinkEndpointId() const;
    
    /**
     * Returns the Source endpoint ID associated for this stream. 
     * Returned endpoint ID is KMaxTUint32 if the source is not yet added.
     * 
     * @return TUint32 Source endpoint identifier.
     */
    TUint32 SourceEndpointId() const;
    
    /**
     * Sets the sink state change flag to ETrue indicating that the sink state
     * change is ready.
     */
    void SinkIsReady();
    
    /**
     * Sets the source state change flag to ETrue indicating that the source 
     * state change is ready.
     */
    void SourceIsReady();
    
    /**
     * Prepares this stream. This function will cause the stream start the
     * preparation process which is asynchronous in nature.
     * 
     * @leave KErrAlreadyExists If the stream is already prepared.
     * @leave KErrNotReady If the stream has not yet been created.
     */
    virtual void PrepareStreamL();
    
    /**
     * Start this stream. This function will cause the stream start the
     * actual streaming. Starting is a asynchronous process.
     * 
     * @leave KErrAlreadyExists If the stream is already prepared.
     * @leave KErrNotReady If the stream has not yet been created.
     */
    virtual void StartStreamL();
    
    /**
     * Pause this (started) stream. This function will cause the stream to 
     * re-start the streaming.
     * 
     * @leave KErrAlreadyExists If the stream is already prepared.
     * @leave KErrNotReady If the stream has not yet been created.
     */
    virtual void PauseStreamL();
    
    /**
     * Resume this (paused) stream. This function will cause the stream to
     * resume streaming.
     * 
     * @leave KErrAlreadyExists If the stream is already prepared.
     * @leave KErrNotReady If the stream has not yet been created.
     */
    virtual void ResumeStreamL();
    
    /**
     * Stop this stream. This function will cause the stream to stop if it is
     * in any of the active states. After calling this function stream must
     * be re-created.
     * 
     * @leave KErrAlreadyExists If the stream is already prepared.
     * @leave KErrNotReady If the stream has not yet been created.
     */
    virtual void StopStreamL();
    
    /**
     * Returns state of the stream.
     * 
     * @return TControlState Stream state.
     */
    TControlState State() const;
    
    /**
     * Returns a reference to the multimedia comms controller interface.
     * 
     * @return CMccInterface Reference to the MCC interface.
     */
    CMccInterface& MccInterface();
    
    /**
     * Function to get the codec of this stream.
     * 
     * @return CMccCodecInformation Reference to the codec of this stream.
     */
    CMccCodecInformation& Codec();
    
    /**
     * Handles a stream category event from MCC.
     * 
     * @param aEvent MCC event to be handled.
     */
    virtual void HandleEvent( const TMccEvent& aEvent );
    
protected:
    
    /**
     * Constructor for performing 1st stage construction.
     * 
     * @param aStateMachine Reference to the state machine.
     * @param aMcc Reference to the MCC.
     * @param aSession ID of the session in which to create this stream.
     * @param aLink ID of the link in which to create this stream.
     */
    CStreamBase( CStreamStateMachine& aStateMachine, CMccInterface& aMcc,
            TUint32 aSession, TUint32 aLink );

    /**
     * Symbian default constructor for performing 2nd stage construction.
     * Deriving classes must call this function. Also codec of the stream is 
     * passed through this function. Note that this function does a deep copy
     * of the given codec using CMccCodecInformation::CloneDetailedL.
     * Base class must also define the type of the stream when calling this
     * function.
     * 
     * @see mmccinterfacedef.h For possible stream types.
     * @see mmcccodecinformation.h For CMccCodecInformation::CloneDetailedL().
     * @leave KErrArgument
     *        If Rng( KMccAudioUplinkStream, aType, KMccDtmfStream ) != ETrue.
     * @leave KErrAlreadyExists If iCodec and iStateMachine have been created.
     * @leave Standard Symbian OS error codes in other cases.
     * @param aCodec Codec of this stream.
     * @param aType Type of this stream.
     */
    virtual void ConstructL( CMccCodecInformation& aCodec, TInt aType );
    
    /**
     * Adds a source for this stream. See mmccinterface.h for different
     * possible data sources.
     * 
     * @pre iSourceEndpoint == KMaxTUint32
     * @post iSourceEndpoint != KMaxTUint32
     * @leave KErrAlreadyExists If iSourceEndpoint != KMaxTUint32.
     * @leave Standard Symbian OS error code in other cases.
     * @param aType Type of the source, @see mmccinterface.h.
     * @param aInitData Initialization data to be passed to the source.
     */
    void AddSourceL( const TUid& aType, const TDesC8& aInitData );
    
    /**
     * Adds a sink for this stream. See mmccinterface.h for different
     * possible data sinks. The endpoint ID of this sink is stored to the
     * internal array.
     * 
     * @leave KErrGeneral If the sink ID generation fails.
     * @leave Standard Symbian OS error code in other cases.
     * @param aType Type of the sink, @see mmccinterface.h.
     * @param aInitData Initialization data to be passed to the sink.
     */
    void AddSinkL( const TUid& aType, const TDesC8& aInitData );
    
    /**
     * Creates this stream. This causes the stream to change its state to
     * 'created' state and it is ready for preparation.
     * 
     * @pre iStreamId == KMaxTUint32
     * @post iStreamId != KMaxTUint32
     * @leave KErrAlreadyExists If this stream has been already created.
     * @leave Standard Symbian OS error code in other cases.
     */
    virtual void CreateStreamL();
    
private:    // Data
    
    /**
     * Codec of this stream.
     * Own.
     */
    CMccCodecInformation* iCodec;
    
    /**
     * Pointer to a current state.
     * Not own
     */
    CStreamStateBase* iState;
    
    /**
     * Current audio control state of this stream.
     */
    TControlState iControlState;
    
    /**
     * Stream state machine.
     */
    CStreamStateMachine& iStateMachine;
    
    /**
     * Reference to MCC interface.
     */
    CMccInterface& iMcc;
    
    /**
     * Session into which this stream belongs to.
     */
    TUint32 iSessionId;
    
    /**
     * Link into which this stream belongs to.
     */
    TUint32 iLinkId;
    
    /**
     * Identifier of this stream.
     */
    TUint32 iStreamId;
    
    /**
     * Type of this stream.
     */
    TInt iStreamType;
    
    /**
     * Source endpoint identifier.
     */
    TUint32 iSourceEndpoint;
    
    /**
     * Sink endpoint identifier.
     */
    TUint32 iSinkEndpoint;
    
    /**
     * Flag to indicate that the link is ready.
     */
    TBool iLinkReady;
    
    /**
     * Flag to indicate that source endpoint state change is ready.
     */
    TBool iSourceReady;
    
    /**
     * Flag to indicate that sink endpoint state change is ready.
     */
    TBool iSinkReady;
    
    /**
     * Flag to indicate RTCP usage status. ETrue by default.
     */
    TBool iRtcpUsage;
    
    };

#endif  // C_CSTREAMBASE_H
